! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!
module gofcoords_mod
!
! GOFLIB coordinate handling routines
! 
  use dal
  use gofconstants_mod
  use gofobsolete_mod ! for now...
  use errorHandling

  implicit none
  
contains
  
  subroutine angsep(dlong1,dlat1,dlong2,dlat2,dang)
!
!   Calculates the angular separation between two celestial bodies
!
!************************ FFORM VERSION 1.2 ************ 30-MAR-90 16:48
!
! author : KOD        date: 06-SEP-1989 16:40
! update : KOD        date: 20-MAR-1990 15:45
! update : KOD        date: 24-OCT-1991 13:57  (sin --> dsin, ...)
! update : BHP        date: 01-OCT-2010 converted to f95, was in 'subsn' in ESAS
!
! status: f95 code tested
!
!   general description
!
!  Subroutine to calculate the angular separation between two
!  celestial bodies. The 'latitude' is defined as to have
!  0 degrees at the equator and +90 degrees at the north pole.
!  Special care is taken for small angles, according to
!    R.W.Sinnott, 'Sky and Telescope', August 1984, p.159.
!
!   call_var.          type I/O description
!
!  DLONG1              R8  I   'longitude 1' (e.g. R.A.; radians)
!  DLAT1               R8  I   'latitude  1' (e.g. Dec.; radians)
!
!  DLONG2              R8  I   'longitude 2' (e.g. R.A.; radians)
!  DLAT2               R8  I   'latitude  2' (e.g. Dec.; radians)
!
!  DANG                R8    O angular separation       (radians)
!  ierr is now handled by sas error task -- bhp
!  IERR                I4    O = 0 no error
!                              = 1 warning
!                              = 2 error
!                              = 3 severe error
!
!   include_block name          description
! eliminated by "use" of constants, etc blocks -- bhp
! R$COMMON:CGENL.CMN          general parameter common block
! R$INCLUDE:IGENL.INC         parameter block with trig. constants
!
!   routines_called    type     description
! the below are now done by sas error handling routine --bhp 
! WRERR               SR      formats error code in standard form!
! WRFLAG              SR      writes to FLAG output stream
!
!   extensions/system calls     description
!
!
!***********************************************************************

    real(kind=double), intent(in) :: dlong1, dlat1, dlong2, dlat2
    real(kind=double), intent(out) :: dang
    real(kind=double) :: d1, d2, dr
    
! bhp: should this be a warning??? might get noisy if so...

    if (max(abs(dlong1),abs(dlong2)).gt.d2pi) then
       write(errstr,*)"angsep: odd dlong1 or dlong2 = ",dlong1, dlong2
       call message(AppMsg,Noisy)
    endif

    if (max(abs(dlat1), abs(dlat2)).gt.dpid2) then 
       write(errstr,*)"angsep: odd dlat1 or dlat2 = ",dlong1, dlong2
       call message(AppMsg,Noisy)
    endif

    d1 = dsin ( 0.5d0 * ( dlat1 -  dlat2 ) )
    d2 = dsin ( 0.5d0 * (dlong1 - dlong2 ) )
    dr = d1 * d1 + d2 * d2 * dcos(dlat1) * dcos(dlat2)
    dang = 2.d0 * datan2 ( dsqrt(dr), dsqrt(1.d0-dr) )

    return

  end subroutine angsep
!
!
!  
  subroutine dradrs(drad,odradrs)
!
!    'Resets' radians to the interval  0 .. 2 * pi
!
!************************ FFORM VERSION 1.2 ************ 30-MAR-90 16:58
!
!  author : KOD        date: 19-JUL-1989 18:31
!  update : KOD        date: 29-MAR-1990 15:08
!  update : BHP        date: 2009 convert to f90
!                            this is actually never called in ESAS
!
!  status: tested
!
!   general description
!  'Resets' radians to the interval  0  .. 2 * pi.
!
!   call_var.          type I/O description
!  DRAD                R8  I   radians (may be negative)
!  ODRADRS             R8    O radians inside the interval  0 .. 2 * pi
!
!   include_block name          description
!  CGENL.CMN          general parameter common block
!  IGENL.INC         parameter block with trig. constants
!
!   routines_called    type     description
!  RESVAL              SR      'resets' periodic variables
!  ERRSTC              SR      continue error stack for one level

    real(kind=double) :: drad, dxnew, odradrs
    real(kind=double) :: dxmin = 0.d0
    real(kind=double) :: dxmax
    real(kind=double) :: dxold

    dxmax = d2pi
    dxold = drad
    call resval(dxmin,dxmax,dxold,dxnew)
    
    odradrs = dxnew
    
    return
  end subroutine dradrs
!
!
!
  subroutine equecl(djd,lmean,ldir,dlold,dbold,dlnew,dbnew)

!   Converts coordinates between equatorial and ecliptical system
!   ONLY called by merge_comp_xmm in ESAS package --bhp
!************************ FFORM VERSION 1.2 ************ 30-MAR-90 17:02
!
! author : KOD        date: 19-JUL-1989 18:31
! update : KOD        date: 29-MAR-1990 15:13
! update : BHP        date: 2012 update to f95, added SAS 
! status: tested
!
!   general description
! Subroutine to convert coordinates between the equatorial and the
! ecliptical system (LII,BII).
!
! To convert the TRUE positions, use the MEAN obliquity of the
! ecliptic (normal case);
! for the APPARENT positions (affected by aberration and nutation)
! use the TRUE obliquity.
! If the equatorial coordinates refer to a standard equinox
! (1950, 2000, ...), the obliquity of the ecliptic for this
! epoch should be used.
!
! (Jean Meeus, 'Astronomical Formulae for Calculators',
!               third edition, 1985, p.43-44)
!
!   call_var.          type I/O description
! DJD                 R8  I   Julian Day for the conversion
!
! LMEAN               L4  I   mean or true obliquity of the ecliptic:
!                               LMEAN = .true.  : mean obliquity
!                               LMEAN = .false. : true obliquity
!
! LDIR                L4  I   transformation direction:
!                               LDIR = .true.  : equ --> ecl
!                               LDIR = .false. : equ <-- ecl
!
! DLOLD               R8  I   old longitude  (radians)
! DBOLD               R8  I   old latitude   (radians)
!
! DLNEW               R8    O new longitude  (radians)
! DBNEW               R8    O new latitude   (radians)
!
! IERR                I4    O = 0 no error
!                             = 1 warning
!                             = 2 error
!                             = 3 severe error
!
!   include_block name          description
! CGENL.CMN          general parameter common block
! IGENL.INC         parameter block with trig. constants
!
!   routines_called    type     description
! EULER1              SR      general coordinate transformation
! JDEPS1              SR      calculates mean/true obl. of the ecl.
! WRERR               SR      formats error code in standard form
! ERRSTC              SR      continue error stack for one level
!
    real(kind=double) :: djd, dlold, dbold, dlnew, dbnew, &
         d1, d2, d3, deps, dphi1, dtheta1, dphi2, dtheta2, dj
    logical :: lmean, ldir, lmn
    
    ! bhp: these blow up later in merge_comp_xmm but should the be warnings???
    if ((dlold .lt. 0.d0) .or. (dlold .gt. d2pi)) then
       write(errstr,*)"equecl: input L is LT 0 or GT TwoPi: ",dlold
       call error("BadLong",trim(errstr))
    endif
    
    if (dabs(dbold) .gt. dpid2) then
       write(errstr,*)"equecl: input B is LT -Pi/2 or GT Pi/2: ",dbold
       call error("BadLat",trim(errstr))
    endif
    
    if ((djd .lt. 0.d0) .or. (djd .gt. 3.d6)) then
       write(errstr,*)"Acceptable range for JD is ~2451179 < JD < ~2470000"
       call message(AppMsg,Noisy)
       write(errstr,*)"equecl: input JD is LT 0 or GT 3E6: ",djd
       call error("badJulDay",trim(errstr))
    endif

    dj = djd
    lmn = lmean
    call jdeps1(dj,lmn,deps)

    d1 = 0.d0
    d2 = deps
    d3 = 0.d0

    dphi1   = dlold
    dtheta1 = dpid2 - dbold
    
    if (ldir) then
       call euler1(d1,d2,d3,dphi1,dtheta1,dphi2,dtheta2)
    else
       call euler1(-d3,-d2,-d1,dphi1,dtheta1,dphi2,dtheta2)
    endif

    dlnew = dphi2
    dbnew = dpid2 - dtheta2

    return
    
  end subroutine equecl
!
!
!
  subroutine equgal(djd,ldir,dlold,dbold,dlnew,dbnew)
!
!    Converts coordinates between equatorial and new galactic system
!
!************************ FFORM VERSION 1.2 ************ 30-MAR-90 17:02
!
! author : KOD        date: 19-JUL-1989 19:45
! update : KOD        date: 20-MAR-1990 14:56
!
! status: tested
!
!   general description
! Subroutine to convert coordinates between the equatorial and the
! new galactic system (LII,BII).
! Parameters extracted from:
!    F.C.Bertiau, 'Programmes for pocket calculators HP-67
!                  and HP-97 in the field of theoretical and
!                  observational astronomy',
!                  1977 Leuven University Press, 'ASTR. COOR. 2'
!
!   call_var.          type I/O description
! DJD                 R8  I   Julian Day for the conversion
!
! LDIR                L4  I   transformation direction:
!                               LDIR = .true.  : equ --> gal II
!                               LDIR = .false. : equ <-- gal II
!
! DLOLD               R8  I   old longitude  (radians)
! DBOLD               R8  I   old latitude   (radians)
!
! DLNEW               R8    O new longitude  (radians)
! DBNEW               R8    O new latitude   (radians)
!
! IERR                I4    O = 0 no error
!                             = 1 warning
!                             = 2 error
!                             = 3 severe error
!
!   include_block name          description
! CGENL.CMN          general parameter common block
! IGENL.INC         parameter block with trig. constants
!
!   routines_called    type     description
! EULER1              SR      general coordinate transformation
! WRERR               SR      formats error code in standard form
! ERRSTC              SR      continue error stack for one level

    real(kind=double) :: djd, dlold, dbold, dlnew, dbnew, &
         d1, d2, d3, dphi1, dtheta1, dphi2, dtheta2, dt
    !  julian day for 1950.01.01 / 00:00:00
    !  real(kind=double) :: djd50 = 2433282.5d0 (gofconstants)
    real(kind=double) :: d11 = 4.92619181375d0
    real(kind=double) :: d21 = 1.09257611174d0
    real(kind=double) :: d31 = 5.70722665400d0
    real(kind=double) :: d12 = 5.82362268000d-7
    real(kind=double) :: d22 = 2.59857718339d-7
    real(kind=double) :: d32 = 0.64223753852d-7
    logical :: ldir

    ! bhp: these blow up later in merge_comp_xmm but should the be warnings???
    if ((dlold .lt. 0.d0) .or. (dlold .gt. d2pi)) then
       write(errstr,*)"equgal: input L is LT 0 or GT TwoPi: ",dlold
       call error("BadLong",trim(errstr))
    endif
    
    if (dabs(dbold) .gt. dpid2) then
       write(errstr,*)"equgal: input B is LT -Pi/2 or GT Pi/2: ",dbold
       call error("BadLat",trim(errstr))
    endif
    
    if ((djd .lt. 0.d0) .or. (djd .gt. 3.d6)) then
       write(errstr,*)"Acceptable range for JD is ~2451179 < JD < ~2470000"
       call message(AppMsg,Noisy)
       write(errstr,*)"equgal: input JD is LT 0 or GT 3E6: ",djd
       call error("badJulDay",trim(errstr))
    endif

    dt = djd - djd1950
    ! bhp: this was a warning for the 5D4, call to WRERR for 5D6
    !      should both be warnings???
    if (dabs(dt).gt.5.d4) then
       write(errstr,*)"equgal: deltaT GT 5D4: ",dt
       call warning("badDeltaT",trim(errstr))
    endif
    if (dabs(dt).gt.5.d6) then
       write(errstr,*)"equgal: deltaT GT 5D6: ",dt
       call error("badDeltaT",trim(errstr))
    endif

    d1 = d11 + d12 * dt
    d2 = d21 + d22 * dt
    d3 = d31 + d32 * dt

    dphi1   = dlold
    dtheta1 = dpid2 - dbold

    if (ldir) then
       call euler1(d1,d2,d3,dphi1,dtheta1,dphi2,dtheta2)
    else
       call euler1(-d3,-d2,-d1,dphi1,dtheta1,dphi2,dtheta2)
    endif

    dlnew = dphi2
    dbnew = dpid2 - dtheta2

    return
  end subroutine equgal
!
!
!
  subroutine euler1(d1,d2,d3,dph1,dth1,dph2,dth2)
!
! bhp: called by equecl and equgal ONLY
!    General transformation of spherical coordinates
!
!************************ FFORM VERSION 1.2 ************ 30-MAR-90 17:03
!
!  author : KOD        date: 15-JUL-1989 18:31
!  update : KOD        date: 20-MAR-1990 14:57
!  update : BHP        date: 2010 f95/XMM-SAS compliant
!    
!  status: tested
!
!   general description
!
!  Subroutine to convert spherical coordinates between coordinate
!  systems which are rotated with respect to each other.
!
!  This subroutine converts the azimuthal and polar angles
!  (DPH1,DTH1), given in mathematical notation and in radians,
!  into the corresponding angles with respect to a coordinate
!  system which orientation is given by the EULER angles (D1,D2,D3)
!  in the following way:
!
!  The original cartesian coordinate system (X0,Y0,Z0) is rotated
!  in three steps:
!
!   1) rotation around the Z0-axis by the angle D1  --->  (X1,Y1,Z1)
!   2) rotation around the X1-axis by the angle D2  --->  (X2,Y2,Z2)
!   3) rotation around the Z2-axis by the angle D3  --->  (X3,Y3,Z3)
!
!  The calculation is accelerated by using differences of angles.
!
!   call_var.          type I/O description
!  D1                  R8  I   EULER angle 1   (radians, -2 pi .. 2 pi)
!  D2                  R8  I   EULER angle 2   (radians, -2 pi .. 2 pi)
!  D3                  R8  I   EULER angle 3   (radians, -2 pi .. 2 pi)
!
!  DPH1                R8  I   original  azimuthal angle (rad, 0..2 pi)
!  DTH1                R8  I   original    polar   angle (rad, 0..pi)
!
!  DPH2                R8    O resulting azimuthal angle (rad, 0..2 pi)
!  DTH2                R8    O resulting   polar   angle (rad, 0..pi)
!
!  IERR                I4    O = 0 no error
!                              = 1 warning
!                              = 2 error
!                              = 3 severe error
!
!   include_block name          description
!  CGENL.CMN          general parameter common block
!  IGENL.INC         parameter block with trig. constants
!
!   routines_called    type     description
!  REPOL1              SR      cartesian to polar coordinate conversion
!  DRADRS              R8      'resets' radians
!  WRERR               SR      formats error code in standard form

    real(kind=double) :: d1, d2, d3, dph1, dph2, dth1, dth2, &
         dst, dct, dh, ds1, dc1, ds2, dc2, dx, dy, dz, &
         dr, dphi, dtheta, idradrs, odradrs
    !bhp: should this be an error???
    if ((max(abs(d1),abs(d2),dabs(d3)) .gt. d2pi) .or. &
         (dph1 .lt. 0.d0)) then
       write(errstr,*)"euler1: odd euler ang d1/d2/d3 = ",d1,d2,d3
       call warning("badEulerAng",trim(errstr))
    endif

    if ((dth1 .lt. 0.d0) .or. (dth1 .gt. dpi)) then
       write(errstr,*)"euler1: odd azm/pol ang dth1/dph1 = ",dth1,dph1
       call error("badAzmPolAng",trim(errstr))
    endif

    dst = sin(dth1)
    dct = cos(dth1)

    dh  = dph1 - d1
    ds1 = sin(dh)
    dc1 = cos(dh)

    ds2 = sin(d2)
    dc2 = cos(d2)

    dx = dst*dc1
    dy = dst*ds1*dc2 + dct*ds2
    dz =-dst*ds1*ds2 + dct*dc2

    call repol1(dx,dy,dz,dr,dphi,dtheta)

    
    if (dabs(dr-1.d0).gt.1.e-6) then
       write(errstr,*)"euler1: odd radius (in m) dr = ",dr
       call warning("badRadius",trim(errstr))
    else
       idradrs = dphi - d3
       call dradrs(idradrs,odradrs)
       dph2 = odradrs
       dth2 = dtheta
    endif

    return
  end subroutine euler1

  subroutine resval(dxmin,dxmax,dxold,dxnew)

!    'Resets' periodic variables to their basic interval
!
!************************ FFORM VERSION 1.2 ************ 30-MAR-90 17:10
!
!  author : KOD        date: 19-JUL-1989 18:31
!  update : KOD        date: 20-MAR-1990 15:10
!
!  status: tested
!
!   general description
!  Subroutine to 'reset' the values of periodically defined
!  variables to their basic interval.
!
!   call_var.          type I/O description
!  DXMIN               R8  I   left  boundary of basic interval
!  DXMAX               R8  I   right boundary of basic interval
!
!  DXOLD               R8  I   value before resetting
!  DXNEW               R8    O value after  resetting
!
!  IERR                I4    O = 0 no error
!                              = 1 warning
!                              = 2 error
!                              = 3 severe error
!
!   include_block name          description
!  CGENL.CMN          general parameter common block (not anymore!)
!
!   routines_called    type     description
!  WRERR               SR      formats error code in standard form (not anymore!)

    real(kind=double) :: dxmin, dxmax, dxold, dxnew, ddiff, dh

    if (dxmin .ge. dxmax) then
       write(errstr,*)"resval: dxmin .ge. dxmax= ",dxmin,dxmax
       call warning("xminGTxmax",trim(errstr))
    endif

    ddiff = dxmax - dxmin
    dh = (dxold-dxmin) / ddiff
    dh = dh - dint(dh)
    if (dh.lt.0.d0) dh = dh + 1.d0
    dxnew = dxmin + dh * ddiff

    return

  end subroutine resval

  subroutine jdeps1(djd,lmean,deps)
!
!   Calculates the MEAN/TRUE obliquity of the ecl. for the Jul. Date
!
!************************ FFORM VERSION 1.2 ************ 30-MAR-90 17:05
!
! author : KOD        date: 29-AUG-1989 17:22
! update : KOD        date: 29-MAR-1990 15:53
! update : BHP        date: 2012 f95/XMM-SAS compliant
! status: tested
!
!   general description
! Subroutine to calculate the MEAN/TRUE obliquity of the ecliptic
! for a given Julian Date, according to
!    Jean Meeus, 'Astronomical Formulae for Calculators',
!                third edition, 1985, p.44,69,70,81.
!
! The parameters for calculating the mean obliquity have been
! taken from 'THE ASTRONOMICAL ALMANACH', 1984, S21.
!
! For APPARENT positions (affected by aberration and nutation)
! use the TRUE obliquity, otherwise the MEAN obliquity.
!
!   call_var.          type I/O description
! DJD                 R8  I   Julian Date
!
! LMEAN               L4  I   mean or true obliquity of the ecliptic:
!                               LMEAN = .true.  : mean obliquity
!                               LMEAN = .false. : true obliquity
!
! DEPS                R8    O obliquity of the ecliptic  (radians)
!
! IERR                I4    O = 0 no error
!                             = 1 warning
!                             = 2 error
!                             = 3 severe error
!
!   include_block name          description
! CGENL.CMN          general parameter common block
!
!   routines_called    type     description
! JDNUT1              SR      calculates the nutation parameters
! WRERR               SR      formats error code in standard form
! ERRSTC              SR      continue error stack for one level

    
    real(kind=double) :: djd, deps, dt, dp, de, dj
    real(kind=double) :: d1 =  0.40909280422d0
    real(kind=double) :: d2 = -2.26965525d-4
    real(kind=double) :: d3 = -2.86040d-9
    real(kind=double) :: d4 =  8.78967d-9
    real(kind=double) :: djd0 = 2451545.d0
    real(kind=double) :: dyear =  36525.d0
    logical :: lmean

    if ((djd .lt. 0.d0) .or. (djd .gt. 3.d6)) then
       write(errstr,*)"Acceptable range for JD is ~2451179 < JD < ~2470000"
       call message(AppMsg,Noisy)
       write(errstr,*)"jdesp1: input JD is LT 0 or GT 3E6: ",djd
       call error("badJulDay",trim(errstr))
    endif

    dt = (djd - djd0) / dyear
    deps = d1 + ( d2 + ( d3 + d4 * dt ) * dt ) * dt

    if (.not.lmean) then
       dj = djd
       call jdnut1(dj,dp,de) ! jdnut1 catches errors
       deps = deps + de
    endif

    return

  end subroutine jdeps1
!
!
!
  subroutine jdnut1(djd,dp,de)
!
!    Calculates the nutation in longitude and in obliquity from the JD
!
!************************ FFORM VERSION 1.2 ************ 30-MAR-90 17:06
!
!  author : KOD        date: 29.08L-1989 14:45
!  update : KOD        date: 20-MAR-1990 15:04
! update : BHP        date: 2012 f95/XMM-SAS compliant
!
!  status: tested
!
!   general description
!  Subroutine to calculate the nutation in longitude and the
!  nutation in obliquity which are needed for the calculation
!  of the APPARENT place of a star and for that of the APPARENT
!  sidereal time, according to
!     Jean Meeus, 'Astronomical Formulae for Calculators',
!                 third edition, 1985, p.69-70 .
!
!   call_var.          type I/O description
!  DJD                 R8  I   Julian Date
!  DP                  R8    O nutation in longitude (radians)
!  DE                  R8    O nutation in obliquity (radians)
!  IERR                I4    O = 0 no error
!                              = 1 warning
!                              = 2 error
!                              = 3 severe error
!
!   include_block name          description
!  CGENL.CMN          general parameter common block
!
!   routines_called    type     description
!  WRERR               SR      formats error code in standard form

    real(kind=double) :: djd, dp, de, dt, dls, dlm, &
         dms, dmm, dom, dh, dls2, dlm2, dom2
    real(kind=double) :: djd0 = 2415020.d0
    real(kind=double) :: dyear =  36525.d0
    real(kind=double) :: df = 4.848136811d-6

    if ((djd .lt. 0.d0) .or. (djd .gt. 3.d6)) then
       write(errstr,*)"Acceptable range for JD is ~2451179 < JD < ~2470000"
       call message(AppMsg,Noisy)
       write(errstr,*)"jdnut1: input JD is LT 0 or GT 3E6: ",djd
       call error("badJulDay",trim(errstr))
    endif
    
    dt = (djd - djd0) / dyear
    dls = 4.8816283d0 + (628.3319506d0 + 5.2883d-6 * dt) * dt
    dlm = 4.7199672d0 + (8399.709144d0 - 1.9775d-5 * dt) * dt
    dms = 6.2565830d0 + (628.3019466d0 - 2.6180d-6 * dt) * dt
    dmm = 5.1680002d0 + (8328.691104d0 + 1.6043d-4 * dt) * dt
    dom = 4.5236020d0 - (33.75714610d0 + 3.6268d-5 * dt) * dt

    dls2 = 2.d0 * dls
    dlm2 = 2.d0 * dlm
    dom2 = 2.d0 * dom

    dh = - (17.2327d0 + 0.01737d0 * dt ) * sin(dom) &
         - ( 1.2729d0 + 0.00013d0 * dt ) * sin(dls2) &
         + 0.2088d0 * sin(dom2) - 0.2037d0 * sin(dlm2) &
         + ( 0.1261d0 - 0.00031d0 * dt) * sin(dms) &
         + 0.0675 * sin(dmm) &
         - (0.0497d0 - 0.00012d0 * dt) * sin(dls2+dms) &
         - 0.0342d0 * sin(dlm2-dom) &
         - 0.0261d0 * sin(dlm2+dmm) &
         + 0.0214d0 * sin(dls2-dms) &
         - 0.0149d0 * sin(dls2-dlm2+dmm) &
         + 0.0124d0 * sin(dls2-dom) &
         + 0.0114d0 * sin(dlm2-dmm)

    dp = dh * df

    dh =   ( 9.2100d0 + 0.00091d0 * dt ) * cos(dom) &
         + ( 0.5522d0 - 0.00029d0 * dt ) * cos(dls2) &
         - 0.0904d0 * cos(dom2) &
         + 0.0884d0 * cos(dlm2) &
         + 0.0216d0 * cos(dls2+dms) &
         + 0.0183d0 * cos(dlm2-dom) &
         + 0.0113d0 * cos(dlm2+dmm) &
         - 0.0093d0 * cos(dls2-dms) &
         - 0.0066d0 * cos(dls2-dom)
    de = dh * df

    return
  end subroutine jdnut1
  !
  ! poidevcal *was* called by pn_back but is no longer ???
  !
  
  subroutine poidevcal (poidev,xm,idum)

    integer :: idum
    real :: alxm, g, sq, t
    real :: gammln, poidev, ran, y
    real :: pi = 3.141592654
    real :: oldm = -1.0
    real(kind=double) em, emt, xm, xmt

    if (xm .lt. 12.0) then
       if (xm .ne. oldm) then
          oldm = xm
          g = exp(-xm)
       endif
       em = -1.0
       t = 1.0
       do while (t .gt. g)
          em = em + 1.0
          call random (ran,idum)
          t = t*ran
          if(ran .gt. 1) then
             write(errstr,*)"poidevcal: returned random number GT 1: ",ran
             call error("badRandom",trim(errstr))
          end if
       enddo
    else
       if (xm .ne. oldm) then
          oldm = xm
          sq = sqrt(2.0*xm)
          alxm = alog(oldm)
          xmt = xm + 1.0
          call gammlncal(gammln,xmt)
          g = xm*alxm - gammln
       endif
       em = -1.0
       do while ((em .lt. 0.0) .or. (ran .gt. t))
          call random (ran,idum)
          y = tan(pi*ran)
          em = sq*y + xm
          if(em .ge. 0.0) then
             em = int(em)
             emt = em + 1.0
             call gammlncal(gammln,emt)
             t = 0.9*(1.0 + y*y)*exp(em*alxm - gammln - g)
             call random (ran,idum)
          endif
       enddo
    endif

    poidev = em

    return
  end subroutine poidevcal
!
!  
!
  subroutine repol1(dx,dy,dz,dr,dphi,dtheta)

!    Converts rectangular coordinates into polar coordinates
!
!************************ FFORM VERSION 1.2 ************ 30-MAR-90 17:10
!
!  author : KOD        date: 15-JUN-1989 18:31
!  update : KOD        date: 20-MAR-1990 15:09
!
!  status: tested
!
!   general description
!  Subroutine to convert the rectangular coordinates (DX,DY,DZ)
!  into the polar coordinates (DR,DPHI,DTHETA) - mathematical
!  notation.
!
!   call_var.          type I/O description
!  DX                  R8  I   x-value
!  DY                  R8  I   y-value
!  DZ                  R8  I   z-value
!
!  DR                  R8    O radius   (same unit as DX,DY,DZ)
!  DPHI                R8    O azimuthal angle  (radians)
!  DTHETA              R8    O  polar    angle  (radians)
!
!   include_block name          description
!  CGENL.CMN          general parameter common block
!  IGENL.INC         parameter block with trig. constants
!
!   routines_called    type     description

    real(kind=double) :: dx, dy, dz, dr, dphi, dtheta, dh

    dr = dsqrt(dx*dx+dy*dy+dz*dz)

    if (dr.eq.0.0d0) then
       dphi   = 0.0d0
       dtheta = 0.0d0
    else
       dtheta = dacos(dz/dr)
       dh = dsqrt(dx*dx+dy*dy)
       if (dh.eq.0.0d0) then
          dphi = 0.0d0
       else
          dphi = dasin(dy/dh)
          if (dx .lt. 0.0d0) dphi = dpi  - dphi
          if (dphi .lt. 0.0d0) dphi = dphi + d2pi
       endif
    endif
    
    return
  end subroutine repol1

end module gofcoords_mod
